module Main where

import Options.Applicative
import System.Process
import Data.List (intercalate)
import Data.List.Split (splitOn)
import Control.Monad

import Types
import Domain
import DomainOperation

import ArgParser hiding (network, disk)

-------------
-- test only
-------------

testDomainXML :: IO ()
testDomainXML = printXML makeDomain {disk=LocalImage "/path/to/foo.img"}

testGetDisk :: IO ()
testGetDisk = do
  d <- getDisk makeDomain {name="vm1", disk=HTTPImage "http://dfs.game.yy.com:8080/download/cirros-0.3.4-x86_64-disk.img"}
  print d
  printXML d

----------------
-- CLI dispatch
----------------

-- | init QVM environment. install all dependencies for running VM.
qvmInit :: IO ()
qvmInit = do
  callCommand "sudo apt-get update"
  callCommand "sudo apt-get install -y openvswitch-switch libvirt-bin python-libvirt qemu qemu-utils virt-top genisoimage"
  callCommand "sudo ovs-vsctl --may-exist add-br br0"
  callCommand "sudo mkdir -p /data/qvm/images/ /data/qvm/instances/"

-- | infer gateway, in test env, gw is always .1
inferGateway :: String -> String
inferGateway ip = intercalate "." (init (splitOn "." ip) ++ ["1"])

-- | create a QvmDomain that matches user's cli options.
domainFromOpts :: CreateOptions -> QvmDomain
domainFromOpts opts =
  makeDomain {name = optName opts
             ,core = optCore opts
             ,ramSizeMB = optRamMB opts
             ,network = optNetwork opts
             ,disk = optDisk opts
             ,ipAddress = ip
             ,gateway = case optGateway opts of
                          Nothing -> if not (ip == Nothing) then
                                         fmap inferGateway ip
                                     else
                                         Nothing
                          x -> x
             ,dns = case optDNS opts of
                      Nothing -> if not (ip == Nothing) then
                                     Just "114.114.114.114"
                                 else
                                     Nothing
                      x -> x} where
      ip = optIPAddress opts

app :: Options -> IO ()
app (Options (Init _)) = qvmInit
app (Options (Create opts)) = do
  let d = domainFromOpts opts
  defineDomain d
  unless (optNoStart opts) (startDomain d)
app (Options (Remove opts)) = removeDomain $ removeOptName opts

main :: IO ()
main = execParser opts >>= app
    where
      opts = info (helper <*> parseOptions)
             ( fullDesc
               <> progDesc "Start KVM VM quickly from the command line. Type qvm COMMAND --help for detail usage."
               <> header "qvm - start KVM VM quickly")
